@@ -35,41 +35,15 @@
 #define _CPP_STRING	1
 
 #include <bits/c++config.h>
-#include <bits/stl_config.h> 
-
-#include <bits/std_climits.h>
-#include <bits/utility.h>
 #include <bits/stl_string_fwd.h>
-#include <bits/std_cstring.h>
-#include <bits/std_cwchar.h>  /* just for MAX_WCHAR etc. */
-#include <bits/std_iosfwd.h>
-#include <bits/fpos.h> //for streamoff, streampos
-#include <bits/char_traits.h>
-#include <bits/std_memory.h>
 #include <bits/std_iterator.h>
-
-#include <bits/stl_alloc.h>
+#include <bits/char_traits.h>
+#include <bits/std_iosfwd.h> // for operators >>, <<, and getline decls
 #include <bits/basic_string.h>
 
 #ifdef _G_NO_TEMPLATE_EXPORT
 # define export
-//# include <bits/std_vector.h>  /* by locale */
-# include <bits/std_cctype.h>  /* by locale */
-# include <bits/std_stdexcept.h> 
-# include <bits/std_typeinfo.h>  /* for bad_cast */
-# include <bits/loccore.h>     /* by string getline */
-# include <bits/ios_base.h>    /* by string getline */
-# include <bits/basic_ios.h>   /* by string getline */
-# include <bits/std_istream.h> /* by string getline */
-# include <bits/std_ostream.h>
-# include <bits/std_streambuf.h>
-# include <bits/std_ctime.h>   /* for struct tm, by locale */
-# include <bits/locfacets.h>   /* by string getline */
-# include <bits/std_algorithm.h> 
 # include <bits/string.tcc> 
-# include <bits/std_limits.h>  // numeric_limits<unsigned long>::digits10
-//# include <bits/loccore.tcc>
-//# include <bits/locfacets.tcc>
 #endif
 
 #endif /* _CPP_STRING */
