@@ -30,11 +30,7 @@
 // Written by Gabriel Dos Reis <Gabriel.Dos-Reis@DPTMaths.ENS-Cachan.Fr>
 
 #ifndef _CPP_VALARRAY
-#define _CPP_VALARRAY
-
-#if ! defined (_GNUG__) && ! defined (__attribute__)
-#define __attribute__(foo) 	// nothing 
-#endif
+#define _CPP_VALARRAY 1
 
 #include <bits/c++config.h>
 #include <bits/std_cstddef.h>
@@ -48,36 +44,48 @@
 
 namespace std {
 
-  template<class _Expression, typename _Tp> class _Meta;
-  template<class _Expr1, class _Expr2, class _Op> class _BinaryExpression;
-  template<class _Expression, class _Operator> class _UnaryExpression;
-  template<class _Expression> class _SliceExpression;
-  template<class _Expression> class _GsliceExpression;
-  template<class _Expression> class _IndirectExpression;
-  template<class _Expression> class _ApplyFunctionWithValue;
-  template<class _Expression> class _ApplyFunctionWithConstRef;
+    template<class _Clos, typename _Tp> class _Expr;
 
-  template<class _Tp> struct _Unary_plus;
-  template<class _Tp> struct _Unary_minus;
-  template<class _Tp> struct _Bitwise_and;
-  template<class _Tp> struct _Bitwise_or;
-  template<class _Tp> struct _Bitwise_xor;  
-  template<class _Tp> struct _Bitwise_not;
-  template<class _Tp> struct _Shift_left;
-  template<class _Tp> struct _Shift_right;
+    template<typename _Tp1, typename _Tp2> class _ValArray;    
+
+    template<template<class> class _Oper,
+        template<class, class> class _Meta, class _Dom> struct _UnClos;
+
+    template<template<class> class _Oper,
+        template<class, class> class _Meta1,
+        template<class, class> class _Meta2,
+        class _Dom1, class _Dom2> class _BinClos;
+
+    template<template<class, class> class _Meta, class _Dom> class _SClos;
+
+    template<template<class, class> class _Meta, class _Dom> class _GClos;
+    
+    template<template<class, class> class _Meta, class _Dom> class _IClos;
+    
+    template<template<class, class> class _Meta, class _Dom> class _ValFunClos;
+
+    template<template<class, class> class _Meta, class _Dom> class _RefFunClos;
+
+    template<class _Tp> struct _Unary_plus;
+    template<class _Tp> struct _Bitwise_and;
+    template<class _Tp> struct _Bitwise_or;
+    template<class _Tp> struct _Bitwise_xor;  
+    template<class _Tp> struct _Bitwise_not;
+    template<class _Tp> struct _Shift_left;
+    template<class _Tp> struct _Shift_right;
   
-  template<class _Tp> class valarray;       // An array of type _Tp
-  class slice;                            // BLAS-like slice out of an array
-  template<class _Tp> class slice_array;
-  class gslice;                           // generalized slice out of an array
-  template<class _Tp> class gslice_array;
-  template<class _Tp> class mask_array;     // masked array
-  template<class _Tp> class indirect_array; // indirected array
+    template<class _Tp> class valarray;   // An array of type _Tp
+    class slice;                          // BLAS-like slice out of an array
+    template<class _Tp> class slice_array;
+    class gslice;                         // generalized slice out of an array
+    template<class _Tp> class gslice_array;
+    template<class _Tp> class mask_array;     // masked array
+    template<class _Tp> class indirect_array; // indirected array
 
 }
 
 #include <bits/valarray_array.h>
-
+#include <bits/valarray_meta.h>
   
 namespace std {
 
@@ -96,8 +104,8 @@ namespace std {
       valarray(const gslice_array<_Tp>&);
       valarray(const mask_array<_Tp>&);
       valarray(const indirect_array<_Tp>&);
-      template<class _Expression>
-      valarray(const _Meta<_Expression,_Tp>& __e);
+      template<class _Dom>
+      valarray(const _Expr<_Dom,_Tp>& __e);
      ~valarray();
 
       // _lib.valarray.assign_ assignment:
@@ -108,34 +116,28 @@ namespace std {
       valarray<_Tp>& operator=(const mask_array<_Tp>&);
       valarray<_Tp>& operator=(const indirect_array<_Tp>&);
 
-      template<class _Expression> valarray<_Tp>&
-      	operator= (const _Meta<_Expression,_Tp>&);
+      template<class _Dom> valarray<_Tp>&
+      	operator= (const _Expr<_Dom,_Tp>&);
 
       // _lib.valarray.access_ element access:
       _Tp                 operator[](size_t) const;
       _Tp&                operator[](size_t);		
       // _lib.valarray.sub_ subset operations:
-      _Meta<_SliceExpression<_Array<_Tp> >, _Tp>
-      	operator[](slice) const;
+      _Expr<_SClos<_ValArray,_Tp>, _Tp> operator[](slice) const;
       slice_array<_Tp>    operator[](slice);
-      _Meta<_GsliceExpression<_Array<_Tp> >, _Tp>
-      	operator[](const gslice&) const;
+      _Expr<_GClos<_ValArray,_Tp>, _Tp> operator[](const gslice&) const;
       gslice_array<_Tp>   operator[](const gslice&);
       valarray<_Tp>     	 operator[](const valarray<bool>&) const;
       mask_array<_Tp>     operator[](const valarray<bool>&);
-      _Meta<_IndirectExpression<valarray<_Tp> >, _Tp>
+      _Expr<_IClos<_ValArray, _Tp>, _Tp>
       	operator[](const valarray<size_t>&) const;
       indirect_array<_Tp> operator[](const valarray<size_t>&);
 
       // _lib.valarray.unary_ unary operators:
-      _Meta<_UnaryExpression<valarray<_Tp>,_Unary_plus<_Tp> >,_Tp>
-        operator+() const;
-      _Meta<_UnaryExpression<valarray<_Tp>,negate<_Tp> >,_Tp>
-        operator-() const;
-      _Meta<_UnaryExpression<valarray<_Tp>,_Bitwise_not<_Tp> >,_Tp>
-        operator~() const;
-      _Meta<_UnaryExpression<valarray<_Tp>,logical_not<_Tp> >,bool>
-        operator!() const;
+      _Expr<_UnClos<_Unary_plus,_ValArray,_Tp>,_Tp>  operator+ () const;
+      _Expr<_UnClos<negate,_ValArray,_Tp>,_Tp> operator- () const;
+      _Expr<_UnClos<_Bitwise_not,_ValArray,_Tp>,_Tp> operator~ () const;
+      _Expr<_UnClos<logical_not,_ValArray,_Tp>,bool> operator! () const;
       
       // _lib.valarray.cassign_ computed assignment:
       valarray<_Tp>& operator*= (const _Tp&);
@@ -159,26 +161,26 @@ namespace std {
       valarray<_Tp>& operator<<=(const valarray<_Tp>&);
       valarray<_Tp>& operator>>=(const valarray<_Tp>&);
 
-      template<class _Expression>
-        valarray<_Tp>& operator*= (const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator/= (const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator%= (const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator+= (const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator-= (const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator^= (const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator|= (const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator&= (const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator<<=(const _Meta<_Expression,_Tp>&);
-      template<class _Expression>
-        valarray<_Tp>& operator>>=(const _Meta<_Expression,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator*= (const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator/= (const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator%= (const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator+= (const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator-= (const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator^= (const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator|= (const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator&= (const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator<<=(const _Expr<_Dom,_Tp>&);
+      template<class _Dom>
+        valarray<_Tp>& operator>>=(const _Expr<_Dom,_Tp>&);
 
       
       // _lib.valarray.members_ member functions:
@@ -192,10 +194,8 @@ namespace std {
 
       valarray<_Tp> shift (int) const;
       valarray<_Tp> cshift(int) const;
-      _Meta<_ApplyFunctionWithValue<valarray<_Tp> >,_Tp>
-        apply(_Tp func(_Tp)) const;
-      _Meta<_ApplyFunctionWithConstRef<valarray<_Tp> >,_Tp>
-        apply(_Tp func(const _Tp&)) const;			
+      _Expr<_ValFunClos<_ValArray,_Tp>,_Tp> apply(_Tp func(_Tp)) const;
+      _Expr<_RefFunClos<_ValArray,_Tp>,_Tp> apply(_Tp func(const _Tp&)) const;
       void resize(size_t __size, _Tp __c = _Tp());
 
   private:
@@ -266,23 +266,23 @@ namespace std {
   template<typename _Tp>
   inline valarray<_Tp>::valarray (const _Tp& __t, size_t __n)
       : _M_size (__n), _M_data (new _Tp[__n])
-  { fill (_M_data, _M_data+_M_size, __t); }
+  { __valarray_fill (_M_data, _M_size, __t); }
 
   template<typename _Tp>
   inline valarray<_Tp>::valarray (const _Tp* __pT, size_t __n)
       : _M_size (__n), _M_data (new _Tp[__n])
-  { copy (__pT, __pT+__n, _M_data); }
+  { __valarray_copy (__pT, __n, _M_data); }
 
   template<typename _Tp>
   inline valarray<_Tp>::valarray (const valarray<_Tp>& __v)
       : _M_size (__v._M_size), _M_data (new _Tp[__v._M_size])
-  { copy (__v._M_data, __v._M_data+_M_size, _M_data); }
+  { __valarray_copy (__v._M_data, _M_size, _M_data); }
 
   template<typename _Tp>
   inline valarray<_Tp>::valarray (const slice_array<_Tp>& __sa)
       : _M_size (__sa._M_sz), _M_data (new _Tp[__sa._M_sz])
   { __valarray_copy (__sa._M_array, __sa._M_sz, __sa._M_stride,
-            _Array<_Tp>(_M_data)); }
+                     _Array<_Tp>(_M_data)); }
 
   template<typename _Tp>
   inline valarray<_Tp>::valarray (const gslice_array<_Tp>& __ga)
@@ -302,8 +302,8 @@ namespace std {
   { __valarray_copy (__ia._M_array, __ia._M_index, 
 		     _Array<_Tp>(_M_data), _M_size); }
 
-  template<typename _Tp> template<class _Expr>
-  inline valarray<_Tp>::valarray (const _Meta<_Expr, _Tp>& __e)
+  template<typename _Tp> template<class _Dom>
+  inline valarray<_Tp>::valarray (const _Expr<_Dom, _Tp>& __e)
       : _M_size (__e.size ()), _M_data (new _Tp[_M_size])
   { __valarray_copy (__e, _M_size, _Array<_Tp>(_M_data)); }
 
@@ -314,7 +314,7 @@ namespace std {
   inline valarray<_Tp>&
   valarray<_Tp>::operator= (const valarray<_Tp>& __v)
   {
-      copy(__v._M_data, __v._M_data + _M_size, _M_data);
+      __valarray_copy(__v._M_data, _M_size, _M_data);
       return *this;
   }
 
@@ -322,7 +322,7 @@ namespace std {
   inline valarray<_Tp>&
   valarray<_Tp>::operator= (const _Tp& __t)
   {
-      fill (_M_data, _M_data + _M_size, __t);
+      __valarray_fill (_M_data, _M_size, __t);
       return *this;
   }
 
@@ -362,20 +362,20 @@ namespace std {
       return *this;
   }
 
-  template<typename _Tp> template<class _Expr>
+  template<typename _Tp> template<class _Dom>
   inline valarray<_Tp>&
-  valarray<_Tp>::operator= (const _Meta<_Expr, _Tp>& __e)
+  valarray<_Tp>::operator= (const _Expr<_Dom, _Tp>& __e)
   {
       __valarray_copy (__e, _M_size, _Array<_Tp>(_M_data));
       return *this;
   }
 
   template<typename _Tp>
-  inline _Meta<_SliceExpression<_Array<_Tp> >, _Tp>
+  inline _Expr<_SClos<_ValArray,_Tp>, _Tp>
   valarray<_Tp>::operator[] (slice __s) const
   {
-      typedef _SliceExpression<_Array<_Tp> > _Expr;
-      return _Meta<_Expr, _Tp> (_Expr (_Array<_Tp>(_M_data), __s));
+      typedef _SClos<_ValArray,_Tp> _Closure;
+      return _Expr<_Closure, _Tp> (_Closure (_Array<_Tp>(_M_data), __s));
   }
 
   template<typename _Tp>
@@ -386,11 +386,11 @@ namespace std {
   }
 
   template<typename _Tp>
-  inline _Meta<_GsliceExpression<_Array<_Tp> >, _Tp>
+  inline _Expr<_GClos<_ValArray,_Tp>, _Tp>
   valarray<_Tp>::operator[] (const gslice& __gs) const
   {
-      typedef _GsliceExpression<_Array<_Tp> > _Expr;
-      return _Meta<_Expr, _Tp> (_Expr (_Array<_Tp>(_M_data), __gs));
+      typedef _GClos<_ValArray,_Tp> _Closure;
+      return _Expr<_Closure, _Tp> (_Closure (_Array<_Tp>(_M_data), __gs));
   }
 
   template<typename _Tp>
@@ -405,7 +405,8 @@ namespace std {
   valarray<_Tp>::operator[] (const valarray<bool>& __m) const
   {
       size_t __s (0);
-      for (size_t __i=0; __i<__m.size(); ++__i)
+      size_t __e (__m.size ());
+      for (size_t __i=0; __i<__e; ++__i)
           if (__m[__i]) ++__s;
       return valarray<_Tp> (mask_array<_Tp> (_Array<_Tp>(_M_data), __s,
                                          _Array<bool> (__m)));
@@ -416,17 +417,18 @@ namespace std {
   valarray<_Tp>::operator[] (const valarray<bool>& __m)
   {
       size_t __s (0);
-      for (size_t __i=0; __i<__m.size(); ++__i)
+      size_t __e (__m.size ());
+      for (size_t __i=0; __i<__e; ++__i)
           if (__m[__i]) ++__s;
       return mask_array<_Tp> (_Array<_Tp>(_M_data), __s, _Array<bool> (__m));
   }
 
   template<typename _Tp>
-  inline _Meta<_IndirectExpression<valarray<_Tp> >, _Tp>
+  inline _Expr<_IClos<_ValArray,_Tp>, _Tp>
   valarray<_Tp>::operator[] (const valarray<size_t>& __i) const
   {
-      typedef _IndirectExpression<valarray<_Tp> > _Expr;
-      return _Meta<_Expr, _Tp> (_Expr (*this, __i));
+      typedef _IClos<_ValArray,_Tp> _Closure;
+      return _Expr<_Closure, _Tp> (_Closure (*this, __i));
   }
 
   template<typename _Tp>
@@ -437,62 +439,6 @@ namespace std {
                                 _Array<size_t> (__i));
   }
 
-#undef _DEFINE_VALARRAY_OPERATOR
-
-#define _DEFINE_VALARRAY_OPERATOR(op, name)				\
-  template<typename _Tp>						\
-  inline _Meta<_UnaryExpression<valarray<_Tp>, name##<_Tp> >, _Tp>	\
-  valarray<_Tp>::operator##op() const					\
-  {									\
-      typedef _UnaryExpression<valarray<_Tp>, name##<_Tp> > _Expr;	\
-      return _Meta<_Expr, _Tp> (*this);					\
-  }
-
-_DEFINE_VALARRAY_OPERATOR(+, _Unary_plus)
-_DEFINE_VALARRAY_OPERATOR(-, negate)
-_DEFINE_VALARRAY_OPERATOR(~, _Bitwise_not)
-
-#undef _DEFINE_VALARRAY_OPERATOR    
-  
-  template<typename _Tp>
-  inline _Meta<_UnaryExpression<valarray<_Tp>, logical_not<_Tp> >, bool>
-  valarray<_Tp>::operator!() const
-  {
-      typedef _UnaryExpression<valarray<_Tp>, logical_not<_Tp> > _Expr;
-      return _Meta<_Expr, bool>(*this);
-  }
-
-#define _DEFINE_VALARRAY_OPERATOR(op, name)				\
-  template<class _Tp>							\
-  inline valarray<_Tp> &						\
-  valarray<_Tp>::operator##op##= (const _Tp &__t)			\
-  {									\
-      _Array_augmented_##name (_Array<_Tp>(_M_data), _M_size, __t);	\
-      return *this;							\
-  }									\
-									\
-  template<class _Tp>							\
-  inline valarray<_Tp> &						\
-  valarray<_Tp>::operator##op##= (const valarray<_Tp> &__v)		\
-  {									\
-      _Array_augmented_##name (_Array<_Tp>(_M_data), _M_size, 		\
-                               _Array<_Tp>(__v._M_data));		\
-      return *this;							\
-  }
-
-_DEFINE_VALARRAY_OPERATOR(+, plus)
-_DEFINE_VALARRAY_OPERATOR(-, minus)
-_DEFINE_VALARRAY_OPERATOR(*, multiplies)
-_DEFINE_VALARRAY_OPERATOR(/, divides)
-_DEFINE_VALARRAY_OPERATOR(%, modulus)
-_DEFINE_VALARRAY_OPERATOR(^, xor)
-_DEFINE_VALARRAY_OPERATOR(&, and)
-_DEFINE_VALARRAY_OPERATOR(|, or)
-_DEFINE_VALARRAY_OPERATOR(<<, shift_left)
-_DEFINE_VALARRAY_OPERATOR(>>, shift_right)
-
-#undef _DEFINE_VALARRAY_OPERATOR    
-
   template<class _Tp>
   inline size_t valarray<_Tp>::size () const { return _M_size; }
 
@@ -516,14 +462,18 @@ _DEFINE_VALARRAY_OPERATOR(>>, shift_right)
   {
       _Tp* const __a = static_cast<_Tp*> (alloca (sizeof(_Tp) * _M_size));
       if (! __n)                          // __n == 0: no shift
-        copy (_M_data, _M_data + _M_size, __a);
-      else if (__n > 0) {                  // __n > 0: shift right
-          copy (_M_data+__n, _M_data + _M_size, __a);
-          fill (__a+_M_size-__n, __a+_M_size, _Tp());
+        __valarray_copy (_M_data, _M_size, __a);
+      else if (__n > 0) {                  // __n > 0: shift left
+          if (__n > _M_size)
+              __valarray_fill(__a, __n, _Tp());
+          else {
+              __valarray_copy (_M_data+__n, _M_size-__n, __a);
+              __valarray_fill (__a+_M_size-__n, __n, _Tp());
+          }
       }
-      else {                             // __n < 0: shift left
-          copy (_M_data, _M_data + _M_size+__n, __a-__n);
-          fill(__a, __a-__n, _Tp());
+      else {                             // __n < 0: shift right
+          __valarray_copy (_M_data, _M_size+__n, __a-__n);
+          __valarray_fill(__a, -__n, _Tp());
       }
       return valarray<_Tp> (__a, _M_size);
   }
@@ -534,14 +484,14 @@ _DEFINE_VALARRAY_OPERATOR(>>, shift_right)
   {
       _Tp* const __a = static_cast<_Tp*> (alloca (sizeof(_Tp) * _M_size));
       if (! __n)                          // __n == 0: no cshift
-        copy(_M_data, _M_data+_M_size, __a);
+        __valarray_copy(_M_data, _M_size, __a);
       else if (__n > 0) {                 // __n > 0: cshift left
-          copy (_M_data, _M_data + __n, __a + _M_size-__n);
-          copy (_M_data + __n, _M_data + _M_size, __a);
+          __valarray_copy (_M_data, __n, __a + _M_size-__n);
+          __valarray_copy (_M_data + __n, _M_size-__n, __a);
       }
       else {                            // __n < 0: cshift right
-          copy (_M_data + _M_size + __n, _M_data + _M_size, __a);
-          copy (_M_data, _M_data + _M_size + __n, __a - __n);
+          __valarray_copy (_M_data + _M_size + __n, -__n, __a);
+          __valarray_copy (_M_data, _M_size + __n, __a - __n);
         }
       return valarray<_Tp> (__a, _M_size);
   }
@@ -555,36 +505,8 @@ _DEFINE_VALARRAY_OPERATOR(>>, shift_right)
           _M_size = __n;
           _M_data = new _Tp[_M_size];
       }
-      fill (_M_data, _M_data+_M_size, __c);
+      __valarray_fill (_M_data, _M_size, __c);
   }
-
-} // std::
-  
-#include <bits/valarray_meta.h>
-
-namespace std {
-
-#define _DEFINE_VALARRAY_OPERATOR(op, name) 				\
-  template<class _Tp> template<class _Expr>				\
-  inline valarray<_Tp> &						\
-  valarray<_Tp>::operator##op##= (const _Meta<_Expr,_Tp> &__e)		\
-  {									\
-      _Array_augmented_##name (_Array<_Tp>(_M_data), __e, _M_size);	\
-      return *this;							\
-  }
-
-_DEFINE_VALARRAY_OPERATOR(+, plus)
-_DEFINE_VALARRAY_OPERATOR(-, minus)
-_DEFINE_VALARRAY_OPERATOR(*, multiplies)
-_DEFINE_VALARRAY_OPERATOR(/, divides)
-_DEFINE_VALARRAY_OPERATOR(%, modulus)
-_DEFINE_VALARRAY_OPERATOR(^, xor)
-_DEFINE_VALARRAY_OPERATOR(&, and)
-_DEFINE_VALARRAY_OPERATOR(|, or)
-_DEFINE_VALARRAY_OPERATOR(<<, shift_left)
-_DEFINE_VALARRAY_OPERATOR(>>, shift_right)
-
-#undef _DEFINE_VALARRAY_OPERATOR    
     
   template<typename _Tp>
   inline _Tp
@@ -601,104 +523,175 @@ _DEFINE_VALARRAY_OPERATOR(>>, shift_right)
   }
   
   template<class _Tp>
-  inline _Meta<_ApplyFunctionWithValue<valarray<_Tp> >,_Tp>
+  inline _Expr<_ValFunClos<_ValArray,_Tp>,_Tp>
   valarray<_Tp>::apply (_Tp func (_Tp)) const
   {
-      typedef _ApplyFunctionWithValue<valarray<_Tp> > _Expr;
-      return _Meta<_Expr,_Tp> (_Expr (*this, func));
+      typedef _ValFunClos<_ValArray,_Tp> _Closure;
+      return _Expr<_Closure,_Tp> (_Closure (*this, func));
   }
 
   template<class _Tp>
-  inline _Meta<_ApplyFunctionWithConstRef<valarray<_Tp> >,_Tp>
+  inline _Expr<_RefFunClos<_ValArray,_Tp>,_Tp>
   valarray<_Tp>::apply (_Tp func (const _Tp &)) const
   {
-      typedef _ApplyFunctionWithConstRef<valarray<_Tp> > _Expr;
-      return _Meta<_Expr,_Tp> (_Expr (*this, func));
+      typedef _RefFunClos<_ValArray,_Tp> _Closure;
+      return _Expr<_Closure,_Tp> (_Closure (*this, func));
   }
 
-#define _DEFINE_VALARRAY_OPERATOR(op, name)				\
+#define _DEFINE_VALARRAY_UNARY_OPERATOR(_Op, _Name)                     \
   template<typename _Tp>						\
-  inline _Meta								\
-    <_BinaryExpression<valarray<_Tp>, valarray<_Tp>, name##<_Tp> >, _Tp> \
-  operator##op (const valarray<_Tp> &__v, const valarray<_Tp> &__w)	\
+  inline _Expr<_UnClos<_Name,_ValArray,_Tp>, _Tp>               	\
+  valarray<_Tp>::operator##_Op() const					\
   {									\
-      typedef _BinaryExpression						\
-          <valarray<_Tp>, valarray<_Tp>, name##<_Tp> > _Expr;		\
-      return _Meta<_Expr, _Tp> (_Expr (__v, __w));			\
-  }									\
-									\
-  template<typename _Tp>						\
-  inline _Meta								\
-    <_BinaryExpression<valarray<_Tp>, _Constant<_Tp>, name##<_Tp> >, _Tp> \
-  operator##op (const valarray<_Tp> &__v, const _Tp &__t)		\
-  {									\
-      typedef _BinaryExpression						\
-        <valarray<_Tp>, _Constant<_Tp>, name##<_Tp> > _Expr;		\
-      return _Meta<_Expr, _Tp> (_Expr (__v, _Constant<_Tp> (__t)));	\
-  }									\
-									\
-  template<typename _Tp>						\
-  inline _Meta								\
-    <_BinaryExpression<_Constant<_Tp>, valarray<_Tp>, name##<_Tp> >, _Tp> \
-  operator##op (const _Tp &__t, const valarray<_Tp> &__v)		\
-  {									\
-      typedef _BinaryExpression						\
-        <_Constant<_Tp>, valarray<_Tp>, name##<_Tp> > _Expr;		\
-      return _Meta<_Expr, _Tp> (_Expr (_Constant<_Tp> (__t), __v));	\
+      typedef _UnClos<_Name,_ValArray,_Tp> _Closure;	                \
+      return _Expr<_Closure, _Tp> (_Closure (*this));			\
   }
 
-_DEFINE_VALARRAY_OPERATOR(+, plus)
-_DEFINE_VALARRAY_OPERATOR(-, minus)
-_DEFINE_VALARRAY_OPERATOR(*, multiplies)
-_DEFINE_VALARRAY_OPERATOR(/, divides)
-_DEFINE_VALARRAY_OPERATOR(%, modulus)
-_DEFINE_VALARRAY_OPERATOR(^, _Bitwise_xor)
-_DEFINE_VALARRAY_OPERATOR(&, _Bitwise_and)
-_DEFINE_VALARRAY_OPERATOR(|, _Bitwise_or)
-_DEFINE_VALARRAY_OPERATOR(<<, _Shift_left)
-_DEFINE_VALARRAY_OPERATOR(>>, _Shift_right)
+    _DEFINE_VALARRAY_UNARY_OPERATOR(+, _Unary_plus)
+    _DEFINE_VALARRAY_UNARY_OPERATOR(-, negate)
+    _DEFINE_VALARRAY_UNARY_OPERATOR(~, _Bitwise_not)
 
-#undef _DEFINE_VALARRAY_OPERATOR
+#undef _DEFINE_VALARRAY_UNARY_OPERATOR
+  
+  template<typename _Tp>
+  inline _Expr<_UnClos<logical_not,_ValArray,_Tp>, bool>
+  valarray<_Tp>::operator!() const
+  {
+      typedef _UnClos<logical_not,_ValArray,_Tp> _Closure;
+      return _Expr<_Closure, bool> (_Closure (*this));
+  }
 
-#define _DEFINE_VALARRAY_OPERATOR(op, name)				\
-  template<typename _Tp>						\
-  inline _Meta								\
-     <_BinaryExpression<valarray<_Tp>, valarray<_Tp>, name##<_Tp> >, bool> \
-  operator##op (const valarray<_Tp> &__v, const valarray<_Tp> &__w)	\
+#define _DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(_Op, _Name)               \
+  template<class _Tp>							\
+  inline valarray<_Tp> &						\
+  valarray<_Tp>::operator##_Op##= (const _Tp &__t)			\
   {									\
-      typedef _BinaryExpression						\
-        <valarray<_Tp>, valarray<_Tp>, name##<_Tp> > _Expr;		\
-      return _Meta<_Expr, bool> (_Expr (__v, __w));			\
+      _Array_augmented_##_Name (_Array<_Tp>(_M_data), _M_size, __t);	\
+      return *this;							\
   }									\
 									\
   template<class _Tp>							\
-  inline _Meta<_BinaryExpression					\
-      <valarray<_Tp>, _Constant<_Tp>, name##<_Tp> >, bool>		\
-  operator##op (const valarray<_Tp> &__v, const _Tp &__t)		\
+  inline valarray<_Tp> &						\
+  valarray<_Tp>::operator##_Op##= (const valarray<_Tp> &__v)		\
   {									\
-      typedef _BinaryExpression						\
-        <valarray<_Tp>, _Constant<_Tp>, name##<_Tp> > _Expr;		\
-      return _Meta<_Expr, bool> (_Expr (__v, _Constant<_Tp> (__t)));	\
+      _Array_augmented_##_Name (_Array<_Tp>(_M_data), _M_size, 		\
+                               _Array<_Tp>(__v._M_data));		\
+      return *this;							\
+  }
+
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(+, plus)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(-, minus)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(*, multiplies)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(/, divides)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(%, modulus)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(^, xor)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(&, and)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(|, or)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(<<, shift_left)
+_DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT(>>, shift_right)
+
+#undef _DEFINE_VALARRAY_AUGMENTED_ASSIGNMENT
+
+
+} // std::
+  
+
+namespace std {
+
+#define _DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(_Op, _Name)          \
+  template<class _Tp> template<class _Dom>				\
+  inline valarray<_Tp> &						\
+  valarray<_Tp>::operator##_Op##= (const _Expr<_Dom,_Tp> &__e)		\
+  {									\
+      _Array_augmented_##_Name (_Array<_Tp>(_M_data), __e, _M_size);	\
+      return *this;							\
+  }
+
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(+, plus)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(-, minus)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(*, multiplies)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(/, divides)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(%, modulus)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(^, xor)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(&, and)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(|, or)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(<<, shift_left)
+_DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT(>>, shift_right)
+
+#undef _DEFINE_VALARRAY_EXPR_AUGMENTED_ASSIGNMENT
+    
+
+#define _DEFINE_BINARY_OPERATOR(_Op, _Name)				\
+  template<typename _Tp>						\
+  inline _Expr<_BinClos<_Name,_ValArray,_ValArray,_Tp,_Tp>, _Tp>        \
+  operator##_Op (const valarray<_Tp> &__v, const valarray<_Tp> &__w)	\
+  {									\
+      typedef _BinClos<_Name,_ValArray,_ValArray,_Tp,_Tp> _Closure;     \
+      return _Expr<_Closure, _Tp> (_Closure (__v, __w));		\
+  }									\
+									\
+  template<typename _Tp>						\
+  inline _Expr<_BinClos<_Name,_ValArray,_Constant,_Tp,_Tp>,_Tp>         \
+  operator##_Op (const valarray<_Tp> &__v, const _Tp &__t)		\
+  {									\
+      typedef _BinClos<_Name,_ValArray,_Constant,_Tp,_Tp> _Closure;	\
+      return _Expr<_Closure, _Tp> (_Closure (__v, __t));	        \
+  }									\
+									\
+  template<typename _Tp>						\
+  inline _Expr<_BinClos<_Name,_Constant,_ValArray,_Tp,_Tp>,_Tp>         \
+  operator##_Op (const _Tp &__t, const valarray<_Tp> &__v)		\
+  {									\
+      typedef _BinClos<_Name,_Constant,_ValArray,_Tp,_Tp> _Closure;     \
+      return _Expr<_Closure, _Tp> (_Closure (__t, __v));        	\
+  }
+
+_DEFINE_BINARY_OPERATOR(+, plus)
+_DEFINE_BINARY_OPERATOR(-, minus)
+_DEFINE_BINARY_OPERATOR(*, multiplies)
+_DEFINE_BINARY_OPERATOR(/, divides)
+_DEFINE_BINARY_OPERATOR(%, modulus)
+_DEFINE_BINARY_OPERATOR(^, _Bitwise_xor)
+_DEFINE_BINARY_OPERATOR(&, _Bitwise_and)
+_DEFINE_BINARY_OPERATOR(|, _Bitwise_or)
+_DEFINE_BINARY_OPERATOR(<<, _Shift_left)
+_DEFINE_BINARY_OPERATOR(>>, _Shift_right)
+
+#undef _DEFINE_BINARY_OPERATOR
+
+#define _DEFINE_LOGICAL_OPERATOR(_Op, _Name)				\
+  template<typename _Tp>						\
+  inline _Expr<_BinClos<_Name,_ValArray,_ValArray,_Tp,_Tp>,bool>        \
+  operator##_Op (const valarray<_Tp> &__v, const valarray<_Tp> &__w)	\
+  {									\
+      typedef _BinClos<_Name,_ValArray,_ValArray,_Tp,_Tp> _Closure;     \
+      return _Expr<_Closure, bool> (_Closure (__v, __w));               \
   }									\
 									\
   template<class _Tp>							\
-  inline _Meta<_BinaryExpression					\
-     <_Constant<_Tp>, valarray<_Tp>, name##<_Tp> >, bool>		\
-  operator##op (const _Tp &__t, const valarray<_Tp> &__v)		\
+  inline _Expr<_BinClos<_Name,_ValArray,_Constant,_Tp,_Tp>,bool>        \
+  operator##_Op (const valarray<_Tp> &__v, const _Tp &__t)		\
   {									\
-      typedef _BinaryExpression						\
-        <_Constant<_Tp>, valarray<_Tp>, name##<_Tp> > _Expr;		\
-      return _Meta<_Expr, bool> (_Expr (_Constant<_Tp> (__t), __v));	\
+      typedef _BinClos<_Name,_ValArray,_Constant,_Tp,_Tp> _Closure;     \
+      return _Expr<_Closure, bool> (_Closure (__v, __t));       	\
+  }									\
+									\
+  template<class _Tp>							\
+  inline _Expr<_BinClos<_Name,_Constant,_ValArray,_Tp,_Tp>,bool>        \
+  operator##_Op (const _Tp &__t, const valarray<_Tp> &__v)		\
+  {									\
+      typedef _BinClos<_Name,_Constant,_ValArray,_Tp,_Tp> _Closure;     \
+      return _Expr<_Closure, bool> (_Closure (__t, __v));	        \
   }
 
-_DEFINE_VALARRAY_OPERATOR(&&, logical_and)
-_DEFINE_VALARRAY_OPERATOR(||, logical_or)
-_DEFINE_VALARRAY_OPERATOR(==, equal_to)
-_DEFINE_VALARRAY_OPERATOR(!=, not_equal_to)
-_DEFINE_VALARRAY_OPERATOR(<, less)
-_DEFINE_VALARRAY_OPERATOR(>, greater)
-_DEFINE_VALARRAY_OPERATOR(<=, less_equal)
-_DEFINE_VALARRAY_OPERATOR(>=, greater_equal)
+_DEFINE_LOGICAL_OPERATOR(&&, logical_and)
+_DEFINE_LOGICAL_OPERATOR(||, logical_or)
+_DEFINE_LOGICAL_OPERATOR(==, equal_to)
+_DEFINE_LOGICAL_OPERATOR(!=, not_equal_to)
+_DEFINE_LOGICAL_OPERATOR(<, less)
+_DEFINE_LOGICAL_OPERATOR(>, greater)
+_DEFINE_LOGICAL_OPERATOR(<=, less_equal)
+_DEFINE_LOGICAL_OPERATOR(>=, greater_equal)
 
 #undef _DEFINE_VALARRAY_OPERATOR
 
