@@ -551,6 +551,18 @@ public:                         // Basic accessors
   const_reference operator[](size_type __n) const 
     { return _M_start[difference_type(__n)]; }
 
+#ifdef __STL_THROW_RANGE_ERRORS
+  void _M_range_check(size_type __n) const {
+    if (__n >= this->size())
+      __stl_throw_range_error("deque");
+  }
+
+  reference at(size_type __n)
+    { _M_range_check(__n); return (*this)[__n]; }
+  const_reference at(size_type __n) const
+    { _M_range_check(__n); return (*this)[__n]; }
+#endif /* __STL_THROW_RANGE_ERRORS */
+
   reference front() { return *_M_start; }
   reference back() {
     iterator __tmp = _M_finish;
@@ -564,7 +576,7 @@ public:                         // Basic accessors
     return *__tmp;
   }
 
-  size_type size() const { return _M_finish - _M_start;; }
+  size_type size() const { return _M_finish - _M_start; }
   size_type max_size() const { return size_type(-1); }
   bool empty() const { return _M_finish == _M_start; }
 
@@ -643,7 +655,7 @@ public:
   // The range version is a member template, so we dispatch on whether
   // or not the type is an integer.
 
-  void assign(size_type __n, const _Tp& __val) {
+  void _M_fill_assign(size_type __n, const _Tp& __val) {
     if (__n > size()) {
       fill(begin(), end(), __val);
       insert(end(), __n - size(), __val);
@@ -654,6 +666,10 @@ public:
     }
   }
 
+  void assign(size_type __n, const _Tp& __val) {
+    _M_fill_assign(__n, __val);
+  }
+
 #ifdef __STL_MEMBER_TEMPLATES
 
   template <class _InputIterator>
@@ -666,7 +682,7 @@ private:                        // helper functions for assign()
 
   template <class _Integer>
   void _M_assign_dispatch(_Integer __n, _Integer __val, __true_type)
-    { assign((size_type) __n, (_Tp) __val); }
+    { _M_fill_assign((size_type) __n, (_Tp) __val); }
 
   template <class _InputIterator>
   void _M_assign_dispatch(_InputIterator __first, _InputIterator __last,
@@ -773,7 +789,10 @@ public:                         // Insert
   iterator insert(iterator __position)
     { return insert(__position, value_type()); }
 
-  void insert(iterator __pos, size_type __n, const value_type& __x); 
+  void insert(iterator __pos, size_type __n, const value_type& __x)
+    { _M_fill_insert(__pos, __n, __x); }
+
+  void _M_fill_insert(iterator __pos, size_type __n, const value_type& __x); 
 
 #ifdef __STL_MEMBER_TEMPLATES  
 
@@ -787,7 +806,7 @@ public:                         // Insert
   template <class _Integer>
   void _M_insert_dispatch(iterator __pos, _Integer __n, _Integer __x,
                           __true_type) {
-    insert(__pos, (size_type) __n, (value_type) __x);
+    _M_fill_insert(__pos, (size_type) __n, (value_type) __x);
   }
 
   template <class _InputIterator>
@@ -978,8 +997,9 @@ void deque<_Tp, _Alloc, __bufsize>
 
 template <class _Tp, class _Alloc, size_t __bufsize>
 void 
-deque<_Tp, _Alloc, __bufsize>::insert(iterator __pos,
-                                      size_type __n, const value_type& __x)
+deque<_Tp, _Alloc, __bufsize>::_M_fill_insert(iterator __pos,
+                                              size_type __n,
+                                              const value_type& __x)
 {
   if (__pos._M_cur == _M_start._M_cur) {
     iterator __new_start = _M_reserve_elements_at_front(__n);
