@@ -28,8 +28,8 @@
  *   You should not attempt to use it directly.
  */
 
-#ifndef _CPP_BITS_STL_VECTOR_H
-#define _CPP_BITS_STL_VECTOR_H 1 
+#ifndef __SGI_STL_INTERNAL_VECTOR_H
+#define __SGI_STL_INTERNAL_VECTOR_H
 
 __STL_BEGIN_NAMESPACE 
 
@@ -154,12 +154,13 @@ class vector : protected _Vector_base<_Tp, _Alloc>
 {
 private:
   typedef _Vector_base<_Tp, _Alloc> _Base;
+  typedef vector<_Tp, _Alloc> vector_type;
 public:
   typedef _Tp value_type;
   typedef value_type* pointer;
   typedef const value_type* const_pointer;
-  typedef value_type* iterator;
-  typedef const value_type* const_iterator;
+  typedef __normal_iterator<pointer, vector_type> iterator;
+  typedef __normal_iterator<const_pointer, vector_type> const_iterator;
   typedef value_type& reference;
   typedef const value_type& const_reference;
   typedef size_t size_type;
@@ -192,10 +193,11 @@ protected:
   void _M_insert_aux(iterator __position);
 
 public:
-  iterator begin() { return _M_start; }
-  const_iterator begin() const { return _M_start; }
-  iterator end() { return _M_finish; }
-  const_iterator end() const { return _M_finish; }
+  iterator begin() { return iterator (_M_start); }
+  const_iterator begin() const
+    { return const_iterator (_M_start); }
+  iterator end() { return iterator (_M_finish); }
+  const_iterator end() const { return const_iterator (_M_finish); }
 
   reverse_iterator rbegin()
     { return reverse_iterator(end()); }
@@ -211,13 +213,25 @@ public:
   size_type max_size() const
     { return size_type(-1) / sizeof(_Tp); }
   size_type capacity() const
-    { return size_type(_M_end_of_storage - begin()); }
+    { return size_type(const_iterator(_M_end_of_storage) - begin()); }
   bool empty() const
     { return begin() == end(); }
 
   reference operator[](size_type __n) { return *(begin() + __n); }
   const_reference operator[](size_type __n) const { return *(begin() + __n); }
 
+#ifdef __STL_THROW_RANGE_ERRORS
+  void _M_range_check(size_type __n) const {
+    if (__n >= this->size())
+      __stl_throw_range_error("vector");
+  }
+
+  reference at(size_type __n)
+    { _M_range_check(__n); return (*this)[__n]; }
+  const_reference at(size_type __n) const
+    { _M_range_check(__n); return (*this)[__n]; }
+#endif /* __STL_THROW_RANGE_ERRORS */
+
   explicit vector(const allocator_type& __a = allocator_type())
     : _Base(__a) {}
 
@@ -269,7 +283,7 @@ public:
   void reserve(size_type __n) {
     if (capacity() < __n) {
       const size_type __old_size = size();
-      iterator __tmp = _M_allocate_and_copy(__n, _M_start, _M_finish);
+      pointer __tmp = _M_allocate_and_copy(__n, _M_start, _M_finish);
       destroy(_M_start, _M_finish);
       _M_deallocate(_M_start, _M_end_of_storage - _M_start);
       _M_start = __tmp;
@@ -283,7 +297,8 @@ public:
   // The range version is a member template, so we dispatch on whether
   // or not the type is an integer.
 
-  void assign(size_type __n, const _Tp& __val);
+  void assign(size_type __n, const _Tp& __val) { _M_fill_assign(__n, __val); }
+  void _M_fill_assign(size_type __n, const _Tp& __val);
 
 #ifdef __STL_MEMBER_TEMPLATES
   
@@ -295,7 +310,7 @@ public:
 
   template <class _Integer>
   void _M_assign_dispatch(_Integer __n, _Integer __val, __true_type)
-    { assign((size_type) __n, (_Tp) __val); }
+    { _M_fill_assign((size_type) __n, (_Tp) __val); }
 
   template <class _InputIter>
   void _M_assign_dispatch(_InputIter __first, _InputIter __last, __false_type)
@@ -345,7 +360,7 @@ public:
       ++_M_finish;
     }
     else
-      _M_insert_aux(__position, __x);
+      _M_insert_aux(iterator(__position), __x);
     return begin() + __n;
   }
   iterator insert(iterator __position) {
@@ -355,7 +370,7 @@ public:
       ++_M_finish;
     }
     else
-      _M_insert_aux(__position);
+      _M_insert_aux(iterator(__position));
     return begin() + __n;
   }
 #ifdef __STL_MEMBER_TEMPLATES
@@ -368,9 +383,8 @@ public:
 
   template <class _Integer>
   void _M_insert_dispatch(iterator __pos, _Integer __n, _Integer __val,
-                          __true_type) {
-    insert(__pos, (size_type) __n, (_Tp) __val);
-  }
+                          __true_type)
+    { _M_fill_insert(__pos, (size_type) __n, (_Tp) __val); }
 
   template <class _InputIterator>
   void _M_insert_dispatch(iterator __pos,
@@ -383,7 +397,10 @@ public:
               const_iterator __first, const_iterator __last);
 #endif /* __STL_MEMBER_TEMPLATES */
 
-  void insert (iterator __pos, size_type __n, const _Tp& __x);
+  void insert (iterator __pos, size_type __n, const _Tp& __x)
+    { _M_fill_insert(__pos, __n, __x); }
+
+  void _M_fill_insert (iterator __pos, size_type __n, const _Tp& __x);
 
   void pop_back() {
     --_M_finish;
@@ -391,14 +408,14 @@ public:
   }
   iterator erase(iterator __position) {
     if (__position + 1 != end())
-      copy(__position + 1, _M_finish, __position);
+      copy(__position + 1, end(), __position);
     --_M_finish;
     destroy(_M_finish);
     return __position;
   }
   iterator erase(iterator __first, iterator __last) {
-    iterator __i = copy(__last, _M_finish, __first);
-    destroy(__i, _M_finish);
+    iterator __i(copy(__last, end(), __first));
+    destroy(__i, end());
     _M_finish = _M_finish - (__last - __first);
     return __first;
   }
@@ -416,10 +433,10 @@ protected:
 
 #ifdef __STL_MEMBER_TEMPLATES
   template <class _ForwardIterator>
-  iterator _M_allocate_and_copy(size_type __n, _ForwardIterator __first, 
+  pointer _M_allocate_and_copy(size_type __n, _ForwardIterator __first, 
                                                _ForwardIterator __last)
 {
-    iterator __result = _M_allocate(__n);
+    pointer __result = _M_allocate(__n);
     __STL_TRY {
       uninitialized_copy(__first, __last, __result);
       return __result;
@@ -427,10 +444,10 @@ protected:
     __STL_UNWIND(_M_deallocate(__result, __n));
   }
 #else /* __STL_MEMBER_TEMPLATES */
-  iterator _M_allocate_and_copy(size_type __n, const_iterator __first, 
+  pointer _M_allocate_and_copy(size_type __n, const_iterator __first, 
                                                const_iterator __last)
   {
-    iterator __result = _M_allocate(__n);
+    iterator __result(_M_allocate(__n));
     __STL_TRY {
       uninitialized_copy(__first, __last, __result);
       return __result;
@@ -482,14 +499,6 @@ operator==(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
          equal(__x.begin(), __x.end(), __y.begin());
 }
 
-// CYGNUS LOCAL 8/18/1998
-template <class _Tp, class _Alloc>
-inline bool 
-operator!=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
-{
-  return !(__x == __y);
-}
-
 template <class _Tp, class _Alloc>
 inline bool 
 operator<(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
@@ -498,30 +507,6 @@ operator<(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
                                  __y.begin(), __y.end());
 }
 
-// CYGNUS LOCAL 8/18/1998
-template <class _Tp, class _Alloc>
-inline bool 
-operator>(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
-{
-  return __y < __x;
-}
-
-// CYGNUS LOCAL 8/18/1998
-template <class _Tp, class _Alloc>
-inline bool 
-operator<=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
-{ 
-  return !(__x > __y);
-}
-
-// CYGNUS LOCAL 8/18/1998
-template <class _Tp, class _Alloc>
-inline bool 
-operator>=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y)
-{
-  return !(__x < __y);
-}
-
 #ifdef __STL_FUNCTION_TMPL_PARTIAL_ORDER
 
 template <class _Tp, class _Alloc>
@@ -530,6 +515,30 @@ inline void swap(vector<_Tp, _Alloc>& __x, vector<_Tp, _Alloc>& __y)
   __x.swap(__y);
 }
 
+template <class _Tp, class _Alloc>
+inline bool
+operator!=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y) {
+  return !(__x == __y);
+}
+
+template <class _Tp, class _Alloc>
+inline bool
+operator>(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y) {
+  return __y < __x;
+}
+
+template <class _Tp, class _Alloc>
+inline bool
+operator<=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y) {
+  return !(__y < __x);
+}
+
+template <class _Tp, class _Alloc>
+inline bool
+operator>=(const vector<_Tp, _Alloc>& __x, const vector<_Tp, _Alloc>& __y) {
+  return !(__x < __y);
+}
+
 #endif /* __STL_FUNCTION_TMPL_PARTIAL_ORDER */
 
 template <class _Tp, class _Alloc>
@@ -539,14 +548,14 @@ vector<_Tp,_Alloc>::operator=(const vector<_Tp, _Alloc>& __x)
   if (&__x != this) {
     const size_type __xlen = __x.size();
     if (__xlen > capacity()) {
-      iterator __tmp = _M_allocate_and_copy(__xlen, __x.begin(), __x.end());
+      pointer __tmp = _M_allocate_and_copy(__xlen, __x.begin(), __x.end());
       destroy(_M_start, _M_finish);
       _M_deallocate(_M_start, _M_end_of_storage - _M_start);
       _M_start = __tmp;
       _M_end_of_storage = _M_start + __xlen;
     }
     else if (size() >= __xlen) {
-      iterator __i = copy(__x.begin(), __x.end(), begin());
+      iterator __i(copy(__x.begin(), __x.end(), begin()));
       destroy(__i, _M_finish);
     }
     else {
@@ -559,7 +568,8 @@ vector<_Tp,_Alloc>::operator=(const vector<_Tp, _Alloc>& __x)
 }
 
 template <class _Tp, class _Alloc>
-void vector<_Tp, _Alloc>::assign(size_t __n, const value_type& __val) {
+void vector<_Tp, _Alloc>::_M_fill_assign(size_t __n, const value_type& __val) 
+{
   if (__n > capacity()) {
     vector<_Tp, _Alloc> __tmp(__n, __val, get_allocator());
     __tmp.swap(*this);
@@ -576,8 +586,8 @@ void vector<_Tp, _Alloc>::assign(size_t __n, const value_type& __val) {
 
 template <class _Tp, class _Alloc> template <class _InputIter>
 void vector<_Tp, _Alloc>::_M_assign_aux(_InputIter __first, _InputIter __last,
-                                       input_iterator_tag) {
-  iterator __cur = begin();
+                                        input_iterator_tag) {
+  iterator __cur(begin());
   for ( ; __first != __last && __cur != end(); ++__cur, ++__first)
     *__cur = *__first;
   if (__first == __last)
@@ -589,19 +599,19 @@ void vector<_Tp, _Alloc>::_M_assign_aux(_InputIter __first, _InputIter __last,
 template <class _Tp, class _Alloc> template <class _ForwardIter>
 void
 vector<_Tp, _Alloc>::_M_assign_aux(_ForwardIter __first, _ForwardIter __last,
-                                  forward_iterator_tag) {
+                                   forward_iterator_tag) {
   size_type __len = 0;
   distance(__first, __last, __len);
 
   if (__len > capacity()) {
-    iterator __tmp = _M_allocate_and_copy(__len, __first, __last);
+    iterator __tmp(_M_allocate_and_copy(__len, __first, __last));
     destroy(_M_start, _M_finish);
     _M_deallocate(_M_start, _M_end_of_storage - _M_start);
     _M_start = __tmp;
     _M_end_of_storage = _M_finish = _M_start + __len;
   }
   else if (size() >= __len) {
-    iterator __new_finish = copy(__first, __last, _M_start);
+    iterator __new_finish(copy(__first, __last, _M_start));
     destroy(__new_finish, _M_finish);
     _M_finish = __new_finish;
   }
@@ -623,27 +633,29 @@ vector<_Tp, _Alloc>::_M_insert_aux(iterator __position, const _Tp& __x)
     construct(_M_finish, *(_M_finish - 1));
     ++_M_finish;
     _Tp __x_copy = __x;
-    copy_backward(__position, _M_finish - 2, _M_finish - 1);
+    copy_backward(__position, iterator(_M_finish - 2), iterator(_M_finish- 1));
     *__position = __x_copy;
   }
   else {
     const size_type __old_size = size();
     const size_type __len = __old_size != 0 ? 2 * __old_size : 1;
-    iterator __new_start = _M_allocate(__len);
-    iterator __new_finish = __new_start;
+    iterator __new_start(_M_allocate(__len));
+    iterator __new_finish(__new_start);
     __STL_TRY {
-      __new_finish = uninitialized_copy(_M_start, __position, __new_start);
-      construct(__new_finish, __x);
+      __new_finish = uninitialized_copy(iterator(_M_start), __position,
+                                        __new_start);
+      construct(__new_finish.base(), __x);
       ++__new_finish;
-      __new_finish = uninitialized_copy(__position, _M_finish, __new_finish);
+      __new_finish = uninitialized_copy(__position, iterator(_M_finish),
+                                        __new_finish);
     }
     __STL_UNWIND((destroy(__new_start,__new_finish), 
-                  _M_deallocate(__new_start,__len)));
+                  _M_deallocate(__new_start.base(),__len)));
     destroy(begin(), end());
     _M_deallocate(_M_start, _M_end_of_storage - _M_start);
-    _M_start = __new_start;
-    _M_finish = __new_finish;
-    _M_end_of_storage = __new_start + __len;
+    _M_start = __new_start.base();
+    _M_finish = __new_finish.base();
+    _M_end_of_storage = __new_start.base() + __len;
   }
 }
 
@@ -660,8 +672,8 @@ vector<_Tp, _Alloc>::_M_insert_aux(iterator __position)
   else {
     const size_type __old_size = size();
     const size_type __len = __old_size != 0 ? 2 * __old_size : 1;
-    iterator __new_start = _M_allocate(__len);
-    iterator __new_finish = __new_start;
+    pointer __new_start = _M_allocate(__len);
+    pointer __new_finish = __new_start;
     __STL_TRY {
       __new_finish = uninitialized_copy(_M_start, __position, __new_start);
       construct(__new_finish);
@@ -679,14 +691,14 @@ vector<_Tp, _Alloc>::_M_insert_aux(iterator __position)
 }
 
 template <class _Tp, class _Alloc>
-void vector<_Tp, _Alloc>::insert(iterator __position, size_type __n, 
-                                const _Tp& __x)
+void vector<_Tp, _Alloc>::_M_fill_insert(iterator __position, size_type __n, 
+                                         const _Tp& __x)
 {
   if (__n != 0) {
     if (size_type(_M_end_of_storage - _M_finish) >= __n) {
       _Tp __x_copy = __x;
-      const size_type __elems_after = _M_finish - __position;
-      iterator __old_finish = _M_finish;
+      const size_type __elems_after = end() - __position;
+      iterator __old_finish(_M_finish);
       if (__elems_after > __n) {
         uninitialized_copy(_M_finish - __n, _M_finish, _M_finish);
         _M_finish += __n;
@@ -704,21 +716,21 @@ void vector<_Tp, _Alloc>::insert(iterator __position, size_type __n,
     else {
       const size_type __old_size = size();        
       const size_type __len = __old_size + max(__old_size, __n);
-      iterator __new_start = _M_allocate(__len);
-      iterator __new_finish = __new_start;
+      iterator __new_start(_M_allocate(__len));
+      iterator __new_finish(__new_start);
       __STL_TRY {
-        __new_finish = uninitialized_copy(_M_start, __position, __new_start);
+        __new_finish = uninitialized_copy(_M_start, __position.base(), __new_start);
         __new_finish = uninitialized_fill_n(__new_finish, __n, __x);
         __new_finish
-          = uninitialized_copy(__position, _M_finish, __new_finish);
+          = uninitialized_copy(__position.base(), _M_finish, __new_finish);
       }
       __STL_UNWIND((destroy(__new_start,__new_finish), 
-                    _M_deallocate(__new_start,__len)));
+                    _M_deallocate(__new_start.base(),__len)));
       destroy(_M_start, _M_finish);
       _M_deallocate(_M_start, _M_end_of_storage - _M_start);
-      _M_start = __new_start;
-      _M_finish = __new_finish;
-      _M_end_of_storage = __new_start + __len;
+      _M_start = __new_start.base();
+      _M_finish = __new_finish.base();
+      _M_end_of_storage = __new_start.base() + __len;
     }
   }
 }
@@ -728,9 +740,9 @@ void vector<_Tp, _Alloc>::insert(iterator __position, size_type __n,
 template <class _Tp, class _Alloc> template <class _InputIterator>
 void 
 vector<_Tp, _Alloc>::_M_range_insert(iterator __pos, 
-                                    _InputIterator __first, 
-                                    _InputIterator __last,
-                                    input_iterator_tag)
+                                     _InputIterator __first, 
+                                     _InputIterator __last,
+                                     input_iterator_tag)
 {
   for ( ; __first != __last; ++__first) {
     __pos = insert(__pos, *__first);
@@ -741,16 +753,16 @@ vector<_Tp, _Alloc>::_M_range_insert(iterator __pos,
 template <class _Tp, class _Alloc> template <class _ForwardIterator>
 void 
 vector<_Tp, _Alloc>::_M_range_insert(iterator __position,
-                                    _ForwardIterator __first,
-                                    _ForwardIterator __last,
-                                    forward_iterator_tag)
+                                     _ForwardIterator __first,
+                                     _ForwardIterator __last,
+                                     forward_iterator_tag)
 {
   if (__first != __last) {
     size_type __n = 0;
     distance(__first, __last, __n);
     if (size_type(_M_end_of_storage - _M_finish) >= __n) {
       const size_type __elems_after = _M_finish - __position;
-      iterator __old_finish = _M_finish;
+      iterator __old_finish(_M_finish);
       if (__elems_after > __n) {
         uninitialized_copy(_M_finish - __n, _M_finish, _M_finish);
         _M_finish += __n;
@@ -770,8 +782,8 @@ vector<_Tp, _Alloc>::_M_range_insert(iterator __position,
     else {
       const size_type __old_size = size();
       const size_type __len = __old_size + max(__old_size, __n);
-      iterator __new_start = _M_allocate(__len);
-      iterator __new_finish = __new_start;
+      iterator __new_start(_M_allocate(__len));
+      iterator __new_finish(__new_start);
       __STL_TRY {
         __new_finish = uninitialized_copy(_M_start, __position, __new_start);
         __new_finish = uninitialized_copy(__first, __last, __new_finish);
@@ -794,15 +806,15 @@ vector<_Tp, _Alloc>::_M_range_insert(iterator __position,
 template <class _Tp, class _Alloc>
 void 
 vector<_Tp, _Alloc>::insert(iterator __position, 
-                           const_iterator __first, 
-                           const_iterator __last)
+                            const_iterator __first, 
+                            const_iterator __last)
 {
   if (__first != __last) {
     size_type __n = 0;
     distance(__first, __last, __n);
     if (size_type(_M_end_of_storage - _M_finish) >= __n) {
       const size_type __elems_after = _M_finish - __position;
-      iterator __old_finish = _M_finish;
+      iterator __old_finish(_M_finish);
       if (__elems_after > __n) {
         uninitialized_copy(_M_finish - __n, _M_finish, _M_finish);
         _M_finish += __n;
@@ -820,8 +832,8 @@ vector<_Tp, _Alloc>::insert(iterator __position,
     else {
       const size_type __old_size = size();
       const size_type __len = __old_size + max(__old_size, __n);
-      iterator __new_start = _M_allocate(__len);
-      iterator __new_finish = __new_start;
+      iterator __new_start(_M_allocate(__len));
+      iterator __new_finish(__new_start);
       __STL_TRY {
         __new_finish = uninitialized_copy(_M_start, __position, __new_start);
         __new_finish = uninitialized_copy(__first, __last, __new_finish);
@@ -848,7 +860,7 @@ vector<_Tp, _Alloc>::insert(iterator __position,
 
 __STL_END_NAMESPACE 
 
-#endif /* _CPP_BITS_STL_VECTOR_H */
+#endif /* __SGI_STL_INTERNAL_VECTOR_H */
 
 // Local Variables:
 // mode:C++
